@@ -10,20 +10,36 @@ import platform
 
 import cv2
 import numpy as np
-from PySide6.QtCore import Qt, QPoint
+import openpyxl
+from PySide6.QtCore import Qt, QPoint, QMimeDatabase
 from PySide6.QtGui import QImage, QCursor
-from PySide6.QtWidgets import QFileDialog, QWidget
+from PySide6.QtWidgets import QFileDialog, QWidget, QMessageBox, QApplication, QPushButton
 import pytesseract
-from maphis.common.download import DownloadDialog
 
+from maphis.common.download import DownloadDialog
 from maphis.common.units import Value, Unit, BaseUnit, CompoundUnit, SIPrefix
 
 
+def create_file_dialog(root_folder: Path, title: str, mode: QFileDialog.FileMode,
+                       extensions: typing.Optional[typing.List[str]], parent: QWidget) -> QFileDialog:
+    file_dialog = QFileDialog(parent)
+    file_dialog.setDirectory(str(root_folder))
+    file_dialog.setFileMode(mode)
+    file_dialog.setWindowTitle(title)
+    file_dialog.setWindowModality(Qt.WindowModality.WindowModal)
+    if extensions is not None:
+        mime_db = QMimeDatabase()
+        mime_types = [mime_db.mimeTypesForFileName(f'file.{ext}') for ext in extensions]
+        filter_strings = [mime_type[0].filterString() for mime_type in mime_types if len(mime_type) > 0]
+        file_dialog.setNameFilters(filter_strings)
+    return file_dialog
+
+
 def choose_folder(parent: QWidget, title = "Open folder", path: typing.Optional[Path] = None) -> typing.Optional[Path]:
     file_dialog = QFileDialog(parent)
     if path is not None:
         file_dialog.setDirectory(str(path))
-    file_dialog.setFileMode(QFileDialog.Directory)
+    file_dialog.setFileMode(QFileDialog.FileMode.Directory)
     file_dialog.setWindowTitle(title)
     file_dialog.setWindowModality(Qt.WindowModal)
     if file_dialog.exec_():
@@ -32,6 +48,42 @@ def choose_folder(parent: QWidget, title = "Open folder", path: typing.Optional[
     return None
 
 
+def let_user_open_path(root_folder: Path=Path.home(), title: str='Open',
+                       mode: typing.Union[typing.Literal['single_file'], typing.Literal['files'], typing.Literal['directory']]='files',
+                       extensions: typing.Optional[typing.List[str]]=None,
+                       parent: typing.Optional[QWidget]=None) -> typing.List[Path]:
+    if mode == 'single_file':
+        file_mode = QFileDialog.FileMode.ExistingFile
+    elif mode == 'files':
+        file_mode = QFileDialog.FileMode.ExistingFiles
+    else:
+        file_mode = QFileDialog.FileMode.Directory
+    file_dialog = create_file_dialog(root_folder, title, file_mode, extensions, parent)
+    file_dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptOpen)
+
+    if file_dialog.exec_():
+        return [Path(selected_file) for selected_file in file_dialog.selectedFiles()]
+    return []
+
+
+def let_user_save_path(root_folder: Path=Path.home(), title: str='Save as',
+                       mode: typing.Union[typing.Literal['single_file'], typing.Literal['directory']]='single_file',
+                       extensions: typing.Optional[typing.List[str]]=None,
+                       parent: typing.Optional[QWidget]=None) -> typing.Optional[Path]:
+    if mode == 'single_file':
+        file_mode = QFileDialog.FileMode.AnyFile
+    else:
+        file_mode = QFileDialog.FileMode.Directory
+
+    file_dialog = create_file_dialog(root_folder, title, file_mode, extensions, parent)
+    file_dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptSave)
+
+    if file_dialog.exec_():
+        if len(file_dialog.selectedFiles()) > 0:
+            return Path(file_dialog.selectedFiles()[0])
+    return None
+
+
 def get_dict_from_doc_str(doc_str: str) -> typing.Dict[str, str]:
     lines = [line for line in doc_str.splitlines() if len(line) > 0 and not line.isspace()]
 
@@ -119,16 +171,18 @@ def get_reference_length(img: np.ndarray) -> typing.Tuple[str, np.ndarray]:
     if platform.system() == 'Windows':
         with importlib.resources.path('maphis.bin', 'Tesseract-OCR') as p:
             pytesseract.pytesseract.tesseract_cmd = str(p / 'tesseract.exe')
-    else:
-        print('am on linux probably')
-    rot = img
-    for i in range(4):
-        rot = cv2.rotate(rot, cv2.ROTATE_90_CLOCKWISE)
-        text = pytesseract.image_to_string(rot)
-        if len(text) == 0:
-            continue
-        if text[0].isdigit() and rot.shape[1] >= rot.shape[0]:
-            return text, rot
+    try:
+        rot = img
+        for i in range(4):
+            rot = cv2.rotate(rot, cv2.ROTATE_90_CLOCKWISE)
+            text = pytesseract.image_to_string(rot)
+            if len(text) == 0:
+                continue
+            if text[0].isdigit() and rot.shape[1] >= rot.shape[0]:
+                return text, rot
+    except pytesseract.TesseractNotFoundError:
+        print('Tessearct is not installed')
+        # TODO show an error message dialog
     return '', img
 
 
@@ -286,3 +340,79 @@ def check_if_tesseract_present() -> bool:
             return True
         else:
             return True
+
+
+def attempt_to_save_with_retries(save_function: typing.Callable[[Path, typing.Any], typing.Any],
+                                 title, mode, extensions, parent, **kwargs) -> typing.Tuple[typing.Optional[Path], typing.Any]:
+    if 'path' not in kwargs:
+        kwargs['path'] = let_user_save_path(Path.home(), title, mode, extensions, parent)
+    if kwargs['path'] is None:
+        return None, None
+    results_saved_or_cancelled = False
+
+    while not results_saved_or_cancelled:
+        try:
+            func_ret_val = save_function(**kwargs)
+            results_saved_or_cancelled = True
+            # show_success_info_dialog(path)
+            return kwargs['path'], func_ret_val
+        except PermissionError:
+            path = show_error_and_ask_for_other_path(kwargs['path'], title, mode, extensions, parent)
+            if path is None:
+                results_saved_or_cancelled = True
+            else:
+                kwargs['path'] = path
+    return None, None
+
+
+def show_success_info_dialog(output_path: Path):
+    diag_box = QMessageBox(QMessageBox.Icon.Information, "Saving complete",
+                       f"The file is stored in {output_path}. Do you wish to open the folder?",
+                       QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
+                       QApplication.activeWindow())
+    diag_box.button(QMessageBox.StandardButton.Yes).clicked.connect(diag_box.accept)
+    diag_box.button(QMessageBox.StandardButton.No).clicked.connect(diag_box.reject)
+
+    if diag_box.exec_() == QMessageBox.DialogCode.Accepted:
+        open_with_default_app(output_path.parent)
+    diag_box.close()
+    diag_box.deleteLater()
+
+
+def show_error_and_ask_for_other_path(old_path: Path, title, mode, extensions, parent) -> typing.Optional[Path]:
+    diag = QMessageBox(parent)
+    diag.setWindowTitle('Cannot save the results')
+    diag.setText(
+        f"The file {old_path} cannot be used for saving (maybe it's opened in another program).")
+    diag.addButton(QMessageBox.StandardButton.Cancel)
+    diag.addButton(QMessageBox.StandardButton.Open)
+    button: QPushButton = diag.button(QMessageBox.StandardButton.Open)
+    button.clicked.connect(diag.accept)
+    button.setText('Save as...')
+
+    if diag.exec_() == QMessageBox.DialogCode.Accepted:
+        return let_user_save_path(old_path.parent, title, mode, extensions, parent)
+    return None
+
+
+def save_text(path: Path, text: str):
+    with open(path, 'w') as f:
+        f.write(text)
+
+
+def catch_exception_and_show_messagebox(title: str, text: str, msg_icon: QMessageBox.Icon, print_exception: bool=False):
+    def msg_decorator(func):
+        def wrapper(*args, **kwargs):
+            try:
+                ret = func(*args, **kwargs)
+                return ret
+            except Exception as e:
+                diag = QMessageBox(msg_icon, title.format_map(kwargs),
+                                   text.format_map(kwargs),
+                                   QMessageBox.StandardButton.Close, parent=QApplication.activeWindow())
+                if print_exception:
+                    diag.setDetailedText(str(e))
+                diag.exec_()
+            return None
+        return wrapper
+    return msg_decorator
\ No newline at end of file
