@@ -9,7 +9,9 @@ import PySide6
 from PySide6.QtCore import Qt, QItemSelection, QModelIndex, QItemSelectionModel, Signal, QUrl, QObject
 from PySide6.QtGui import QIcon, QPixmap, QColor, QBrush
 from PySide6.QtWidgets import QDialog, QTreeWidgetItem, QTreeWidget, QAbstractItemView, QDialogButtonBox, QVBoxLayout, \
-    QWidget, QGroupBox, QFileDialog
+    QWidget, QGroupBox, QFileDialog, QMessageBox
+from common.utils import let_user_save_path, attempt_to_save_with_retries, save_text, let_user_open_path, \
+    catch_exception_and_show_messagebox
 
 from maphis.common.common import Info
 from maphis.common.label_hierarchy import Node, LabelHierarchy
@@ -126,6 +128,7 @@ class MeasurementAssignDialog(QDialog):
 
         self.ui.btnAssignmentRemove.clicked.connect(self.remove_assignment_by_selection)
 
+        self.ui.btnDemoSelectColorAndTolerance.hide()
         self.ui.btnDemoSelectColorAndTolerance.clicked.connect(self.demo_select_color_and_tolerance)
 
         self.ui.buttonBox.button(QDialogButtonBox.Apply).setEnabled(False)
@@ -166,19 +169,29 @@ class MeasurementAssignDialog(QDialog):
         self.ui.btnLoadConfiguration.clicked.connect(self._load_assignments)
 
     def _save_current_assignment(self):
-        result: typing.Tuple[QUrl, typing.Any] = QFileDialog.getSaveFileUrl(self, 'Save configuration as...')
-        if result[0].isEmpty():
+        save_path = let_user_save_path(extensions=['json'], parent=self)
+        if save_path is None:
             return
         assignments_json = [prop_assignment.to_dict() for prop_assignment in self.property_assignments.values() if len(prop_assignment.regions)]
-        save_path = pathlib.Path(result[0].toLocalFile())
-        with open(save_path, 'w') as f:
-            json.dump(assignments_json, f, indent=2)
+
+        config_str = json.dumps(assignments_json, indent=2)
+        attempt_to_save_with_retries(save_text, title='Save configuration as...', mode='single_file', path=save_path, extensions=['json'], parent=self, text=config_str)
 
     def _load_assignments(self):
-        result: typing.Tuple[QUrl, typing.Any] = QFileDialog.getOpenFileUrl(self, 'Open configuration as...')
-        if result[0].isEmpty():
+        # result: typing.Tuple[QUrl, typing.Any] = QFileDialog.getOpenFileUrl(self, 'Open configuration as...')
+        # if result[0].isEmpty():
+        #     return
+        load_path = let_user_open_path(pathlib.Path.home(), title='Open configuration file',
+                                       mode='single_file', extensions=['json'], parent=self)
+        if len(load_path) == 0:
             return
-        load_path = pathlib.Path(result[0].toLocalFile())
+        load_path = load_path[0]
+        self.try_to_load_assignments_from(load_path=load_path)
+
+    @catch_exception_and_show_messagebox('Loading configuration file failed',
+                                         'Could not load configuration from {load_path}',
+                                         QMessageBox.Icon.Critical, print_exception=True)
+    def try_to_load_assignments_from(self, load_path: pathlib.Path):
         with open(load_path) as f:
             assignments_dicts = json.load(f)
         # TODO clear current assignments
