@@ -7,6 +7,7 @@ import PySide6
 import numpy as np
 from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt
 from PySide6.QtGui import QVector3D, QColor, QPixmap
+from common.icon_storage import IconStorage
 from skimage.color import hsv2rgb
 
 from maphis import qimage2ndarray
@@ -45,11 +46,13 @@ class MeasurementsTableModel(QAbstractTableModel):
         self.displayed_property_key: str = ''
         self._display_intensity_in_color: bool = True  # color value will either be displayed as QColor(True) or numbers(False)
         self.current_label_name: str = ''
+        self._invalid_property_bg_color: QColor = QColor.fromString('#FFEB9C')
 
     def rowCount(self, parent:PySide6.QtCore.QModelIndex=QModelIndex()) -> int:
         if self.state.storage is None:
             return 0
-        return self.state.storage.image_count
+        # return self.state.storage.image_count
+        return self.state.image_list_model.rowCount(parent)
         pass
 
     def columnCount(self, parent:PySide6.QtCore.QModelIndex=QModelIndex()) -> int:
@@ -60,7 +63,8 @@ class MeasurementsTableModel(QAbstractTableModel):
     def data(self, index:PySide6.QtCore.QModelIndex, role:int=Qt.DisplayRole) -> typing.Any:
         if self.state.storage is None or index.column() >= self.columnCount():
             return None
-        photo = self.state.storage.get_photo_by_idx(index.row(), load_image=False)
+        # photo = self.state.storage.get_photo_by_idx(index.row(), load_image=False)
+        photo = self.state.image_list_model.fetch_photo(index.row())
         label = self.prop_tuple_list[index.column()][0]
         props = photo[self.current_label_name].get_region_props(label)
 
@@ -124,6 +128,10 @@ class MeasurementsTableModel(QAbstractTableModel):
             if prop.prop_type == PropertyType.Vector:
                 if prop.vector_viz is None:
                     viz = vector_to_img(prop.value[0], (128, 24))
+                    # if not prop.up_to_date:
+                    #     viz = np.dstack((viz,) * 3)
+                    #     mask = np.array(viz[:, :, 0] > 200)[:, :, np.newaxis]
+                    #     viz = np.where(mask, [[*self._invalid_property_bg_color.toTuple()[:3]]], [[0, 0, 0]])
                     prop.vector_viz = QPixmap.fromImage(qimage2ndarray.array2qimage(viz))
                 return prop.vector_viz
         if role == MeasurementsTableRole.RegionProperty:
@@ -132,6 +140,12 @@ class MeasurementsTableModel(QAbstractTableModel):
             if prop.prop_type == PropertyType.NDArray:
                 val: np.ndarray = prop.value[0]
                 return f'Double click to explore {val.shape[1:]} matrices for {",".join(prop.val_names)}'
+            elif prop.prop_type in {PropertyType.IntensityHSV, PropertyType.Intensity}:
+                return ', '.join([f'{v[0]:.3f} {v[1]}' for v in zip(prop.value[0], prop.val_names)])
+        # if role == Qt.ItemDataRole.BackgroundRole and not prop.up_to_date:
+        #     return QColor(180, 100, 0)
+        # if role == Qt.ItemDataRole.DecorationRole and not prop.up_to_date:
+        #     return IconStorage.instance()['caution']
         return None
 
     def headerData(self, section:int, orientation:PySide6.QtCore.Qt.Orientation, role:int=Qt.DisplayRole) -> typing.Any:
@@ -141,7 +155,15 @@ class MeasurementsTableModel(QAbstractTableModel):
             if orientation == Qt.Horizontal:
                 return self.column_names[section]
             if orientation == Qt.Vertical:
-                return self.state.storage.image_names[section]
+                # return self.state.storage.image_names[section]
+                return self.state.image_list_model.image_names[section]
+        if orientation == Qt.Orientation.Vertical and (role == Qt.ItemDataRole.DecorationRole or role == Qt.ItemDataRole.ToolTipRole):
+            photo = self.state.image_list_model.fetch_photo(section)
+            if not photo['Labels'].has_valid_measurements():
+                if role == Qt.ItemDataRole.DecorationRole:
+                    return IconStorage.instance()['caution']
+                if role == Qt.ItemDataRole.ToolTipRole:
+                    return 'Measurements not updated for the latest segmentation.'
         return None
 
     def display_property(self, prop_key: str):
@@ -150,7 +172,8 @@ class MeasurementsTableModel(QAbstractTableModel):
         header_labels = set()
         labels = set()
         for i in range(self.state.storage.image_count):
-            photo = self.state.storage.get_photo_by_idx(i, load_image=False)
+            # photo = self.state.storage.get_photo_by_idx(i, load_image=False)
+            photo = self.state.image_list_model.fetch_photo(i)
             #header_labels = header_labels.union({self.state.colormap.label_names[prop.label] for prop in photo['Labels'].prop_list})
             labels = labels.union({prop.label for prop in photo['Labels'].prop_list})
         self.labels = list(labels)
@@ -159,13 +182,13 @@ class MeasurementsTableModel(QAbstractTableModel):
         #self.header_labels = [self.state.colormap.label_names[label] for label in self.labels]
         self.header_labels = [lab_hierarchy.nodes[label].name for label in self.labels]
         start = self.index(0, 0)
-        end = self.index(self.rowCount()-1, self.columnCount() - 1)
+        end = self.index(self.rowCount() - 1, self.columnCount() - 1)
         self.dataChanged.emit(start, end)
 
     def display_intensity_in_color(self, in_color: bool = True):
         self._display_intensity_in_color = in_color
         start = self.index(0, 0)
-        end = self.index(self.rowCount()-1, self.columnCount() - 1)
+        end = self.index(self.rowCount() - 1, self.columnCount() - 1)
         self.dataChanged.emit(start, end)
 
     def update_model(self):
@@ -197,7 +220,8 @@ class MeasurementsTableModel(QAbstractTableModel):
             #    col_name = f':{name}'
             self.column_names.append(col_name)
         start = self.index(0, 0)
-        end = self.index(self.rowCount()-1, self.columnCount() - 1)
+        # end = self.index(self.rowCount()-1, self.columnCount() - 1)
+        end = self.index(self.state.storage.image_count - 1, self.columnCount() - 1)
         self.dataChanged.emit(start, end)
 
 
