@@ -13,7 +13,7 @@ from PySide6.QtWidgets import QWidget, QTableWidgetItem, QTableView, QProgressDi
 from maphis.common.blocking_operation import ProgressReport
 from maphis.common.label_hierarchy import Node
 from maphis.common.label_image import RegionProperty, PropertyType, LabelImg
-from maphis.common.photo import Photo, UpdateEvent, UpdateContext
+from maphis.common.photo import Photo, UpdateEvent, UpdateContext, LabelImageUpdateType
 from maphis.common.plugin import PropertyComputation, ActionContext, GeneralAction
 from maphis.common.regions_cache import RegionsCache
 from maphis.common.state import State
@@ -37,7 +37,7 @@ class MeasurementsViewer(QWidget):
         self.ui.setupUi(self)
 
         self.state = state
-        self.state.storage_changed.connect(self._handle_storage_changed)
+        # self.state.storage_changed.connect(self.set_new_storage)
 
         self.action_context: ActionContext = action_context
 
@@ -61,7 +61,7 @@ class MeasurementsViewer(QWidget):
             action.setData(gen_action)
             # self._export_menu.addAction(action)
         self._default_export_action: QAction = self._export_menu.actions()[0]
-        self.ui.btnExport.setText(self._default_export_action.text())
+        self.ui.btnExport.setText('Export measurements')
 
         self.ui.btnExport.setMenu(self._export_menu)
 
@@ -94,7 +94,6 @@ class MeasurementsViewer(QWidget):
         self._handle_export_action_triggered(self._default_export_action)
 
     def _handle_export_action_triggered(self, action: QAction):
-        self.ui.btnExport.setText(action.text())
         self._default_export_action = action
         if action.data() is None:
             return
@@ -103,7 +102,7 @@ class MeasurementsViewer(QWidget):
         self.action_context.units = self.state.units
         gen_action(self.state, self.action_context)
 
-    def _handle_storage_changed(self, new_strg: typing.Optional[Storage], old_strg: typing.Optional[Storage]):
+    def set_new_storage(self, new_strg: typing.Optional[Storage], old_strg: typing.Optional[Storage]):
         self.ui.cmbLabelImages.blockSignals(True)
         if old_strg is not None:
             old_strg.update_photo.disconnect(self._handle_photo_update)
@@ -125,7 +124,13 @@ class MeasurementsViewer(QWidget):
     def _handle_photo_update(self, update: UpdateEvent):
         if update.update_context != UpdateContext.LabelImg:
             return
-        self.update_measurements_view()
+        index = self.state.image_list_model.image_paths.index(update.photo.image_path)
+        if update.update_obj.update_type in {LabelImageUpdateType.PropertiesValid, LabelImageUpdateType.PropertiesInvalid}:
+            self.model.headerDataChanged.emit(Qt.Orientation.Vertical, index, index)
+        elif update.update_obj.update_type in {LabelImageUpdateType.PropertyUpdate, LabelImageUpdateType.PropertyRemoved, LabelImageUpdateType.PropertyAdded}:
+            model_index_first = self.state.image_list_model.index(index, 0)
+            model_index_last = self.state.image_list_model.index(index, self.model.columnCount() - 1)
+            self.model.dataChanged.emit(model_index_first, model_index_last)
 
     def register_computation(self, comp: PropertyComputation):
         pass
@@ -183,6 +188,8 @@ class MeasurementsViewer(QWidget):
         self.ui.tableView.selectionModel().selectionChanged.connect(self.enable_delete_recompute)
         self.ui.btnExport.setEnabled(True)
         self.ui.tableView.resizeColumnsToContents()
+        # self.ui.tableView.setSortingEnabled(True)
+        # self.ui.tableView.sortByColumn(0, Qt.SortOrder.AscendingOrder)
 
     def enable_delete_recompute(self):
         enable = len(self.ui.tableView.selectedIndexes()) > 0
@@ -262,13 +269,46 @@ class MeasurementsViewer(QWidget):
         prop: typing.Optional[RegionProperty] = index.data(MeasurementsTableRole.RegionProperty)
         if prop is None:
             return
-        if prop.prop_type != PropertyType.NDArray:
+        # if prop.prop_type != PropertyType.NDArray and prop.prop_type != PropertyType.Vector:
+        #     return
+
+        if prop.prop_type == PropertyType.NDArray:
+            table_widget, text = self.create_ndarray_table(prop)
+        elif prop.prop_type == PropertyType.Vector:
+            table_widget, text = self.create_vector_table(prop)
+        else:
             return
+
         if self.nd_browser is not None:
+            widgets: typing.List[QWidget] = []
+            for i in range(self.nd_browser.count()):
+                widgets.append(self.nd_browser.widget(i))
+            self.nd_browser.clear()
+            for w in widgets:
+                w.deleteLater()
             size = self.nd_browser.size()
         else:
+            self.nd_browser = QTabWidget()
             size = None
-        self.nd_browser = QTabWidget()
+
+        self.nd_browser.addTab(table_widget, 'Table view')
+
+        photo: Photo = index.data(MeasurementsTableRole.Photo)
+        self.nd_browser.setWindowTitle(f'{photo.image_name} {self.state.label_hierarchy.nodes[prop.label].name}:{prop.info.name}')
+        # lay = QVBoxLayout()
+        text_edit = QTextEdit()
+        text_edit.setReadOnly(True)
+        text_edit.zoomIn(3)
+        text_edit.setText(text)
+        self.nd_browser.addTab(text_edit, 'Raw view')
+        self.nd_browser.setWindowModality(Qt.ApplicationModal)
+        self.nd_browser.show()
+        if size is not None:
+            self.nd_browser.resize(size)
+        # self.nd_browser.res
+        table_widget.adjustSize()
+
+    def create_ndarray_table(self, prop: RegionProperty) -> typing.Tuple[QWidget, str]:
 
         table_widget = QWidget()
         table_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
@@ -296,6 +336,7 @@ class MeasurementsViewer(QWidget):
                     table.setItem(r, c, titem)
             tables_layout.addWidget(QLabel(prop.val_names[i]))
             tables_layout.addWidget(table)
+            tables_layout.addSpacing(32)
             table.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
             # table.adjustSize()
 
@@ -303,20 +344,29 @@ class MeasurementsViewer(QWidget):
             text += str(prop.value[0][i])
             text += '\n'
 
-        self.nd_browser.addTab(table_widget, 'Table view')
+        return table_widget, text
 
-        self.nd_browser.setWindowTitle(f'{self.state.label_hierarchy.nodes[prop.label].name} - {prop.info.name}')
-        lay = QVBoxLayout()
-        text_edit = QTextEdit()
-        text_edit.setReadOnly(True)
-        text_edit.zoomIn(3)
-        text_edit.setText(text)
-        self.nd_browser.addTab(text_edit, 'Raw view')
-        self.nd_browser.setWindowModality(Qt.ApplicationModal)
-        self.nd_browser.show()
-        if size is not None:
-            self.nd_browser.setFixedSize(size)
-        table_widget.adjustSize()
+    def create_vector_table(self, prop: RegionProperty) -> typing.Tuple[QWidget, str]:
+        widget = QWidget()
+        layout = QVBoxLayout()
+
+        widget.setLayout(layout)
+
+        widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+
+        table_widget = QTableWidget(1, prop.num_vals)
+        layout.addWidget(table_widget)
+
+        table_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+
+        table_widget.setVerticalHeaderLabels([prop.info.name])
+        table_widget.setHorizontalHeaderLabels(prop.val_names)
+
+        for i in range(prop.num_vals):
+            table_item = QTableWidgetItem(f'{prop.value[0][i]:.2f}')
+            table_widget.setItem(0, i, table_item)
+
+        return widget, ', '.join([str(v) for v in prop.value[0]])
 
     def delete_measurements(self):
         indexes: typing.List[QModelIndex] = self.ui.tableView.selectedIndexes()
